import os

class DirStructure(object):
	def __init__(self):
		self.preference_dir = os.path.join(os.getenv("HOME"), ".rescuetime")

		self.preference_file = os.path.join(self.preference_dir, "preferences")

		# After each upload, the windows data collecter requests a CoreData.xsd file
		# This program doesn't use it, but does check if the file ever changes
		# If the file ever changes, the program may need to be updated.
		# The latest code_data file is stored here.
		self.xsd_file_storage_path = os.path.join(self.preference_dir, "latest_core_data.xsd")

		# Each file in PREPARED_DIR is given a unique sequential id.  This file
		# stores the id between sessions.
		self.next_upload_file = os.path.join(self.preference_dir, "next_upload_file")

		# Stores uploaded files which have not been sent yet, either because they
		# failed their last upload, or the program was asked to quickly shut down.
		self.prepared_dir = os.path.join(self.preference_dir, "prepared")

		# Other than the prepared_dir, all of these directories just store
		# debugging data.

		# Stores the replys we get from rescuetime.com
		self.reply_dir = os.path.join(self.preference_dir, "replys")
		# Stores old uploads that succeeded
		self.finished_dir = os.path.join(self.preference_dir, "finished")
		# Stores old uploads that failed 
		self.failed_dir = os.path.join(self.preference_dir, "failed")
 
		self.tmp_dir = os.path.join(self.preference_dir, "tmp")
		self.lock_file = os.path.join(self.tmp_dir, "lock")
		self.latest_time_obj_file = os.path.join(self.tmp_dir, 
		    "latest_time_obj")

		self.disable_tracking_file = os.path.join(self.preference_dir, "disable")

		self.create_dir(self.preference_dir)
		self.create_dir(self.finished_dir)
		self.create_dir(self.prepared_dir)
		self.create_dir(self.failed_dir)
		self.create_dir(self.reply_dir)
		self.create_dir(self.tmp_dir)

		try:
			os.stat(self.next_upload_file)
		except OSError:
			f = file(self.next_upload_file, "w")
			f.write("0")
			f.close()
		
		try:
			os.stat(self.preference_file)
		except OSError:
			f = file(self.preference_file, "w")
			f.close()

	def create_dir(self, dir):
		"""
		Creates the given directory, if it doesn't already exist.
		"""
		try:
			os.stat(dir)
		except OSError:
			os.mkdir(dir, 0700)

	def tracking(self):
		try:
			os.stat(self.disable_tracking_file)
			return False
		except OSError:
			return True


