#!/usr/bin/env python

from pkg_resources import resource_filename
import pygtk
import time
pygtk.require('2.0')
import RescueTimeUploader.uploader
import RescueTimeUploader.logindialog

import gtk
import gnomeapplet
import gobject
import sys
import os
from os.path import abspath, join, dirname, exists
import gnome
import signal
from datetime import datetime

import subprocess
import traceback
from threading import Thread

import sys
#out = open('/tmp/rescuetimeapplet.log', 'w+')
#sys.stdout = out
#sys.stderr = out

#print "hi from process", os.getpid()
#out.flush()

gobject.threads_init()

def TraceBackDialog(text):
	try:
		print "Traceback:", text
		d = gtk.Dialog('Exception loading applet',buttons=(("Close", 1)))
		d.vbox.pack_start(gtk.Label(text))
		d.show_all()
		result = d.run()
	except:
		traceback.print_exc()

class RescueTimeApplet:
	def on_button_pressed(self, widget, event, data):
		if event.button == 1:
			self.trackitem.set_active(not self.trackitem.active)
			self.trackitem_callback(self.trackitem)
	
	def aboutitem_callback(self, widget, event):
		subprocess.Popen("gnome-open https://launchpad.net/rescuetime-linux-uploader", shell=True)

	def dashboarditem_callback(self, widget, event):
		subprocess.Popen("gnome-open http://rescuetime.com/dashboard", shell=True)

	def preferencesitem_callback(self, widget, event):
		if not hasattr(self, "uploader"):
			return
		subprocess.Popen("gnome-open "  + self.uploader.dir_structure.preference_file, shell=True)

	def trackitem_callback(self, widget):
		if not hasattr(self, "uploader"):
			return
		self.uploader.options.tracking = widget.active

		self.image.clear()
		imagename = resource_filename(__name__, "images/rescuetime_22.png")
		if not self.uploader.options.tracking:
			imagename =  resource_filename(__name__, "images/rescuetime_disabled_22.png")
		print "imagename:", imagename
		#out.flush()
		self.image.set_from_file(imagename)	
		self.image.show()
		#self.button.set_image(self.image)


	def timeout_callback(self, event=None):
		try:
			if self.notifier:
				self.notifierlabel.set_label(self.get_display_text())
			else:
				self.image.set_tooltip_text(self.get_display_text())
		except:
			TraceBackDialog(traceback.format_exc(3))
		finally:
			return 1
	
	def __init__(self, applet, iid,notifier=False):
		print "initializing"
		#out.flush()
		gnome.init("RescueTime","1.0")
		print "gnome.init"
		#out.flush()

		self.applet = applet
		self.applet.connect("destroy", self.cleanup)

		self.notifier = notifier

		print "applet connected"
		#out.flush()
		self.init_gtk()

		print "gtk initialized"
		#out.flush()

		
		applet.show_all()

		gobject.timeout_add_seconds(5, self.timeout_callback)
		gobject.timeout_add_seconds(1, self.start_uploader)

	def get_credentials(self):
		if not hasattr(self, "uploader"):
			return
		email_address, password = self.uploader.retrieve_credentials()

		tried = False
		d = None
		while not self.uploader.test_login(email_address, password):
			if not tried:
				helpful_hint = "Please enter your RescueTime login information."
			else:
				helpful_hint = "Login failed.  Please try again."
			d = RescueTimeUploader.logindialog.LoginDialog(helpful_hint, "Enter login information", buttons=(("Login", 1)))
			ret = d.run()
			if ret == -4:
				sys.exit(1)
			else:
				tried = True
				email_address, password = d.get_data()
			d.destroy()

		self.uploader.options.save_preferences(email_address, password)


	def start_uploader(self):
		try:
			self.uploader = RescueTimeUploader.uploader.Uploader(headless=True)
		except:
			TraceBackDialog(traceback.format_exc(3))
			sys.exit(1)
			#out.flush()
		print "uploader created"
		#out.flush()
		self.uploader_thread = Thread(target=self.uploader.main)
		print "thread created"
		#out.flush()
		self.uploader_thread.setDaemon(True)
		self.uploader_thread.start()
		print "thread started"
		self.get_credentials()
		#out.flush()
		return 0

	
	def cleanup(self, *args, **kwards):
		if hasattr(self, "uploader"):
			self.uploader.shutdown()
		sys.exit(0)

	def make_transparent(self, widget):
		widget.connect("realize", lambda widget: widget.window.set_back_pixmap(None, True))
		#widget.window.set_back_pixmap (None, True)
	
	def get_display_text(self):
		if not hasattr(self, "uploader"):
			return "Loading..."
		if not self.uploader.options.tracking:
			return "Not tracking"
		latest_time_obj = self.uploader.get_latest_time_obj()
		if latest_time_obj == None or latest_time_obj.app_name == None\
		    or latest_time_obj.app_name.strip().lower() == "none":
			return "Idle"
		total_time = datetime.today() - latest_time_obj.start_time
		app_name = latest_time_obj.app_name
		if hasattr(latest_time_obj, "extended_info"):
			app_name = latest_time_obj.extended_info
			if app_name.startswith("http://www."):
				app_name = app_name[11:]
			elif app_name.startswith("https://www."):
				app_name = app_name[12:]
			elif app_name.startswith("http://"):
				app_name = app_name[7:]
			elif app_name.startswith("https://"):
				app_name = app_name[8:]
	
		if total_time.seconds < 60:
			time_str = " (<1 min)"
		elif total_time.seconds < 120:
			time_str = " (1 min)"
		elif total_time.seconds < 3600:
			time_str = " (%i mins)" % (total_time.seconds // 60)
		else:
			time_str = " (%i:%i)" % (total_time.seconds // 3600,
			                         (total_time.seconds % 3600) // 60)

		return app_name[:10] + time_str

	def init_gtk(self):
		"""
		Sets up all the gtk widgets.
		"""
		#self.button = gtk.Button()
		#self.applet.add(self.button)

	

		self.image = gtk.Image()

		imagename = resource_filename(__name__, "images/rescuetime_22.png")
		print "imagename:", imagename
		#out.flush()
		self.image.set_from_file(imagename)
		self.make_transparent(self.image)
		self.image.show()
		#self.button.add(self.image)

		self.eventbox = gtk.HBox(spacing=5)
		self.make_transparent(self.eventbox)
		#self.eventbox.connect("realize", self.make_transparent)
		self.eventbox.add(self.image)

		if self.notifier:
			self.notifierlabel = gtk.Label("Loading...")
			self.make_transparent(self.notifierlabel)
			self.notifierlabel.show()
			self.eventbox.add(self.notifierlabel)
		self.applet.add(self.eventbox)
		self.make_transparent(self.applet)


		self.eventbox.connect("button-press-event", self.on_button_pressed, None)
		#self.button.set_events(self.button.get_events() | gtk.gdk.BUTTON_PRESS_MASK)
		#self.button.set_focus_on_click(False)


		self.trackitem = gtk.CheckMenuItem("Tracking", True)
		#self.trackitem.set_active(self.uploader.options.tracking)
		self.trackitem.connect("activate", self.trackitem_callback)
		self.trackitem.show()
		
		menuxml = """
		<popup name="button3">
		 <menuitem name="About" verb="About" label="_About"
			   pixtype="stock" pixname="gtk-about"/>
		 <menuitem name="Preferences" verb="Preferences" label="_Preferences"
			   pixtype="stock" pixname="gtk-properties"/>
		<menuitem name="Dashboard" verb="Dashboard" label="Open Dashboard"
			   pixtype="stock" pixname="gtk-home"/>		</popup>
		"""

		menuverbs = [ 
			("Preferences", self.preferencesitem_callback),
			("About", self.aboutitem_callback),
			("Dashboard", self.dashboarditem_callback),
		]

		self.applet.setup_menu(menuxml, menuverbs)
		#print self.applet.menu

#		applet.connect("destroy", self.cleanup)



def factory_factory(notifier=False):
	print "Making a factory!"
	def sample_factory(applet, iid):
		print "I am in the factory!"
		##out.flush()
		rapplet = RescueTimeApplet(applet, iid, notifier)

		print rapplet
		#out.flush()

		applet.connect("destroy", rapplet.cleanup)

		return True
	print "Made a factory!"
	return sample_factory

def handle_signal(a, b):
	sys.exit(0)

#print "hi"
##out.flush()

signal.signal(signal.SIGTERM, handle_signal)
signal.signal(signal.SIGHUP, handle_signal)

#print "he"
##out.flush()
def gnome_panel_window():
	main_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
	main_window.set_title("Rescuetime Panel Applet")
	main_window.connect("destroy", gtk.main_quit) 
	app = gnomeapplet.Applet()
	factory_factory("notify" in sys.argv)(app, None)
	app.reparent(main_window)
	main_window.show_all()
	try:
		gtk.main()
	except KeyboardInterrupt:
		sys.exit()
	sys.exit()

def gnome_panel(notifier=False):
	try:
		if notifier:
			idname = "OAFIID:GNOME_RescueTimeDurationNotifierApplet_Factory"
		else:
			idname = "OAFIID:GNOME_RescueTimeDurationApplet_Factory"
		a = gnomeapplet.bonobo_factory(idname,
				   gnomeapplet.Applet.__gtype__,
				   "hello", "0", factory_factory(notifier))
	except KeyboardInterrupt:
		sys.exit()
	except:
		TraceBackDialog(traceback.format_exc(3))
