from getopt import gnu_getopt, GetoptError
import sys
import datetime
import gnomekeyring as gkey

# Options/Initialization
class Options(object):
	def __init__(self, dir_structure):
		self.dir_structure = dir_structure
		# Options from command line
		self.supported_short_options = ""
		self.supported_long_options = ["force=", "debug", "noupload", "notrack"]

		self.debug = False
		self.upload = True
		self.tracking = True
		self.force_active_application = None

		try:
			options, leftover = gnu_getopt(sys.argv[1:], 
		                               self.supported_short_options,
					       self.supported_long_options)
			for option in options:
				if option[0] == "--debug":
					self.debug = True
				if option[0] == "--force":
					self.force_active_application=option[1]
				if option[0] == "--noupload":
					self.upload = False 
				if option[0] == "--notrack":
					self.tracking = False
		except GetoptError:
			pass

		self.scan_interval_seconds = 5
		self.upload_interval_minutes = 30
		self.max_idle_minutes = 2
		self.applet_notifier_minimum_seconds = 60

		self.reload_preferences()

		self.upload_interval = datetime.timedelta(0, self.upload_interval_minutes*60)
	
	def reload_preferences(self):
		# Options from preference file
		f = file(self.dir_structure.preference_file)
		data = f.readlines()
		f.close()

		for line in data:
			LL = line.split(":")
			if LL[0].strip() == "scan interval seconds":
				self.scan_interval_seconds = int(LL[1])
			elif LL[0].strip() == "upload interval minutes":
				self.upload_interval_minutes = int(LL[1])
			elif LL[0].strip() == "max idle minutes":
				self.max_idle_minutes = int(LL[1])
			elif LL[0].strip() == "notifier applet minimum seconds between context switches":
				self.applet_notifier_minimum_seconds = int(LL[1])


	def save_preferences(self, email_address, password):
		"""
		Saves all the current preferences in ~/.rescuetime/preferences,
		in a format that can be read by the uploader when it next runs.
		"""
		# Dump out the newly added email and password 
		f = file(self.dir_structure.preference_file, "w")
		f.write("email: " + email_address + "\n")
		f.write("scan interval seconds: " + str(self.scan_interval_seconds) + "\n")
		f.write("upload interval minutes: " + str(self.upload_interval_minutes) + "\n")
		f.write("max idle minutes: " + str(self.max_idle_minutes) + "\n")
		f.write("notifier applet minimum seconds between context switches: " + str(self.applet_notifier_minimum_seconds) + "\n")
		f.close()
		attrs = { "server": "http://www.rescuetime.com",
			  "user": email_address}
                if gkey.is_available():
                    gkey.item_create_sync(gkey.get_default_keyring_sync(),
                                          gkey.ITEM_NETWORK_PASSWORD, "rescuetime", attrs, password, True)
                else:
                    print( "gnomekeyring not available." )

